@@ -86,6 +86,7 @@ my $spec_parser =
 #
 # _Iterator_doc_cache : {} doc_key => $doc cache documents that are lazily read
 #                       by asking for a non-indexed doc element from an iterator
+# _Iterator_doing_lazy_doc_eval
 
 sub new {
   my ( $class, %args ) = @_;
@@ -102,6 +103,7 @@ sub new {
 
     ( $self->{_Iterator_columns_lst}, $self->{_Iterator_columns_pos} ) =
       $self->_make_columns_lsts ( $args{fields} );
+    $self->{_Iterator_doing_lazy_doc_eval} = 0;
 
     my $cspec_arg;
     if ( $args{collection_spec} and $args{sort_spec} ) {
@@ -327,14 +329,14 @@ sub textsearch_not_found {
 }
 
 sub retrieve_doc {
-  my $self = shift();
-  return XML::Comma::Doc->retrieve( $self->doc_key() );
+  my ($self, %args) = @_;
+  return XML::Comma::Doc->retrieve( $self->doc_key(), %args );
 }
 
 sub read_doc {
-  my $self = shift();
+  my ($self, %args) = @_;
   return $self->{_Iterator_doc_cache}->{$self->doc_key} ||
-    XML::Comma::Doc->read( $self->doc_key() );
+    XML::Comma::Doc->read( $self->doc_key(), %args );
 }
 
 # alias doc_(read && retrieve) to (read && retrieve)_doc 
@@ -492,6 +494,7 @@ sub _make_collection_spec {
           $self->{_Iterator_index}->get_dbh_reader()->quote ( "%$partial%" );
 
       } elsif ( $type eq 'binary table' ) {
+#NOTE: this is NOT dead code
         push @binary_tables, $table_name;
         die "can't use NOT with binary-tables-type collection '$name'\n"
           if $NOT;
@@ -795,8 +798,11 @@ sub iterator_dispatch {
   # look to the doc if we didn't find anything in the index
   my $prev_error = $@;  undef $@;
   my $doc;
+  die $prev_error if($self->{_Iterator_doing_lazy_doc_eval});
   eval {
+    $self->{_Iterator_doing_lazy_doc_eval} = 1;
     $doc = $self->read_doc();
+    $self->{_Iterator_doing_lazy_doc_eval} = 0;
     $value = $doc->$m();
   };
 
